using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Dapfor.Net.Diagnostics;
using Dapfor.Net.Grid.BasketViewer.BusinessLevel;
using Dapfor.Net.Ui;
using Dapfor.Net.Grid.BasketViewer.Properties;

namespace Dapfor.Net.Grid.BasketViewer
{
    public partial class Form1 : Form
    {
        private readonly Basket _basket = new Basket();
        private readonly Random _r = new Random();
        private string _regExpression;
        readonly Timer _timer = new Timer();


        public Form1()
        {
            InitializeComponent();

            BuildData();


            grid.Appearance.VerticalLines = true;
            grid.Appearance.HorizontalLines = true;
            grid.Appearance.HorizontalFullSizeLines = false;

            grid.DataSource = _basket.InstrumentClasses;
            grid.Hierarchy.ButtonBehaviour = ExpansionButtonBehaviour.HideAlways;
            grid.Hierarchy.ButtonWidth = 0;

            grid.Rows.ExpandAll();

            grid.RowUpdated += delegate(object sender, GridRowUpdateEventArgs e)
            {
                //When share's price is changed, the % Market part for all rows should computed
                if(e.DataField.Id == "Price")
                {
                    grid.Headers[0]["MarketPartPercent"].InvalidateCells();
                }

                if(e.Row.DataObject is Share)
                {
                    Share share = (Share) e.Row.DataObject;
                    Color color = Color.FromArgb(80, share.Price > share.LastPrice ? Color.Green : Color.Red);
                    e.Row[e.DataField.Id].Highlight(TimeSpan.FromMilliseconds(1000), color);
                }
            };

            grid.PaintCell += delegate(object sender, PaintCellEventArgs e)
            {
                //Prevent from vertical lines painting in 1st and 2nd levels
                if(e.Cell.Row.Level < 2)
                {
                    e.Parts ^= e.Parts & (PaintPart.VerticalLines | PaintPart.HorizontalLines);    
                }
                
                if (e.Cell.Row.DataObject is Share && e.Cell.Column != null)
                {
                    Share share = (Share) e.Cell.Row.DataObject;
                    switch (e.Cell.Column.Id)
                    {
                        case "Price":
                            e.ImageSettings.Alignment = ContentAlignment.MiddleLeft;
                            if (share.Price > share.LastPrice)
                            {
                                e.Image = Resources.arrow_up_green;
                            }
                            if (share.Price < share.LastPrice)
                            {
                                e.Image = Resources.arrow_down_red;
                            }
                            e.Appearance.BackColor = Color.FromArgb(255, 255, 215, 0);
                            break;

                        case "LastPrice":
                            e.Appearance.BackColor = Color.FromArgb(255, 255, 228, 196);
                            break;

                        case "History":
                            e.Parts &= e.Parts ^ PaintPart.Highlighting;
                            e.PaintBackground();
                            e.Parts &= e.Parts ^ (PaintPart.Background | PaintPart.Text);
                            PaintHistory(share, e.Cell, e.Graphics);
                            break;
                    }
                }
            };

            grid.PaintRow += delegate(object sender, PaintRowEventArgs e)
            {
                //Set background colors:
                if (e.Row.DataObject is Industry)
                {
                    e.Appearance.BackColor = Color.FromArgb(255, 0, 0, 139);
                    e.Appearance.ForeColor = Color.White;
                }

                if (e.Row.DataObject is InstrumentClass)
                {
                    e.Appearance.BackColor = Color.FromArgb(255, 239, 0, 0);
                    e.Appearance.ForeColor = Color.White;
                }


                e.PaintAll();
                e.Handled = true;

                //Do post painting for the whole row
                Industry industry = e.Row.DataObject as Industry;
                if (industry != null)
                {
                    e.Graphics.DrawString(industry.IndustryName, e.Font, Brushes.White, e.Row.Bounds);
                }

                InstrumentClass instrumentClass = e.Row.DataObject as InstrumentClass;
                if (instrumentClass != null)
                {
                    e.Graphics.DrawString(instrumentClass.InstrumentClassName, e.Font, Brushes.White, e.Row.Bounds);
                }
            };

            grid.Filter = new Filter(delegate(Row row)
            {
                if (!string.IsNullOrEmpty(_regExpression))
                {
                    if(row.DataObject is Share)
                    {
                        return !IsMatch((Share) row.DataObject);
                    }
                    if(row.DataObject is Industry)
                    {
                        //Hide industry if no shares match to regular expression
                        Industry industry = (Industry) row.DataObject;
                        foreach (Share share in industry.Shares)
                        {
                            if(IsMatch(share)) return false;
                        }
                        return true;
                    }
                }
                return false;
            });

            _timer.Tick += OnRandomUpdate;
            _timer.Interval = 100;
            grid.Highlighting.Fading = false;
        }

        private void PaintHistory(Share share, Cell cell, Graphics graphics)
        {
            if (share.History != null && share.History.Count > 0 && cell.VirtualBounds.Height > 0)
            {
                decimal min = 0;
                decimal max = 0;
                foreach (decimal d in share.History)
                {
                    min = min > 0 ? Math.Min(min, d) : d;
                    max = Math.Max(max, d);
                }
                decimal vRatio = (max - min) / cell.VirtualBounds.Height;

                List<Point> points = new List<Point>();
                for (int i = 0; i < share.History.Count; ++i)
                {
                    decimal v = share.History[i];
                    int dy = (vRatio > 0 ? (int)((v - min) / vRatio) - 1 : cell.VirtualBounds.Height / 2);
                    dy = Math.Max(0, dy);
                    int y = cell.VirtualBounds.Y + dy;
                    int x = cell.VirtualBounds.Right - ((i + 1) * cell.VirtualBounds.Width / Share.MaxHistoryLength);
                    if (points.Count == 0)
                    {
                        points.Add(new Point(cell.VirtualBounds.Right, y));
                    }
                    points.Add(new Point(x, y));
                }
                using (Pen p = new Pen(Color.Blue))
                {
                    graphics.DrawLines(p, points.ToArray());
                }
            }

        }

        private void BuildData()
        {
            InstrumentClass instrumentClass = new InstrumentClass("Shares", _basket);

            Industry industry = new Industry("Automobile & Parts", instrumentClass);
            industry.AddShare(new Share("CONTINENTAL AG", "DE0005439004", industry, 77.1m, 661));
            industry.AddShare(new Share("MICHELIN (CGDE)B", "FR0000121261", industry, 66.38m, 1662));
            instrumentClass.AddIndustry(industry);

            industry = new Industry("Utilities", instrumentClass);
            industry.AddShare(new Share("RWE AG", "DE0007037129", industry, 31.698m, 1750));
            instrumentClass.AddIndustry(industry);

            industry = new Industry("Technology", instrumentClass);
            industry.AddShare(new Share("ATOS ORIGIN", "FR0000051732", industry, 53.28m, 1362));
            industry.AddShare(new Share("INGENICO", "FR0000125346", industry, 39.4675m, 1886));
            industry.AddShare(new Share("SAP AG", "DE0007164600", industry, 57.54m, 1382));
            instrumentClass.AddIndustry(industry);


            _basket.AddIndustryType(instrumentClass);
        }

        private void btnUpdatePrice_Click(object sender, EventArgs e)
        {
            InstrumentClass instrumentClass = _basket.InstrumentClasses[_r.Next(_basket.InstrumentClasses.Count)];
            Industry industry = instrumentClass.Industries[_r.Next(instrumentClass.Industries.Count)];
            Share share = industry.Shares[_r.Next(industry.Shares.Count)];

            //Update price of random-selected share
            decimal newCoeff = (decimal) (1.05 - _r.NextDouble()/10);
            share.Price *= newCoeff;
        }

        private void addItems_Click(object sender, EventArgs e)
        {
            InstrumentClass instrumentClass = new InstrumentClass("Class " + _basket.InstrumentClasses.Count, _basket);

            Industry industry = new Industry("Industry 1", instrumentClass);
            for(int i = 0; i < 100; ++i)
            {
                industry.AddShare(new Share("Share " + i, "XX000000" + i, industry, (decimal)(1.05 - _r.NextDouble()/10) * 100m, _r.Next(1000, 10000)));
            }
            instrumentClass.AddIndustry(industry);
            _basket.AddIndustryType(instrumentClass);

            grid.Rows.ExpandAll();

            //Invalidate market part in all cells
            grid.Headers[0]["MarketPartPercent"].InvalidateCells();
            
        }

        #region Filter routines

        private void tbISIN_TextChanged(object sender, EventArgs e)
        {
            _regExpression = tbISIN.Text;
            _regExpression = _regExpression.Replace("*", ".*");
            _regExpression = _regExpression.Replace("?", ".");
            _regExpression = _regExpression.Trim();
            if (_regExpression == ".*") _regExpression = string.Empty;

            grid.FilterRefresh();
        }

        private bool IsMatch(Share share)
        {
            try
            {
                return Regex.IsMatch(share.Isin, "^" + _regExpression, RegexOptions.Singleline | RegexOptions.IgnoreCase);
            }
            catch
            {
                return false;
            }
        }

        #endregion

        private void cbRandomUpdates_CheckedChanged(object sender, EventArgs e)
        {
            if(cbRandomUpdates.Checked)
            {
                _timer.Start();    
            }
            else
            {
                _timer.Stop();
            }
        }

        private void OnRandomUpdate(object sender, EventArgs e)
        {
            btnUpdatePrice_Click(this, EventArgs.Empty);
        }

        private void btnInspector_Click(object sender, EventArgs e)
        {
            InspectorForm inspector = new InspectorForm();
            inspector.Show(this);
        }

    }
}